==> kpogre-1.2.7.tar.gz: kpogre-1.2.7/kpogre/DbObjects/kpgtreeitem.cpp <==
/***************************************************************************
                          kpgtreeitem.cpp  -  description
                             -------------------
    begin                : Ne led 11 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgtreeitem.h"

#include <qtable.h>

#include <kiconloader.h>
#include <kdebug.h>

#include "../kpglinklabel.h"

QPixmap * KPGTreeItem::m_pIconServer = 0;
QPixmap * KPGTreeItem::m_pIconTablespaceSys = 0;
QPixmap * KPGTreeItem::m_pIconTablespaceUsr = 0;
QPixmap * KPGTreeItem::m_pIconDatabase = 0;
QPixmap * KPGTreeItem::m_pIconDatabaseSys = 0;
QPixmap * KPGTreeItem::m_pIconDatabaseUsr = 0;
QPixmap * KPGTreeItem::m_pIconSchema = 0;
QPixmap * KPGTreeItem::m_pIconSchemaSys = 0;
QPixmap * KPGTreeItem::m_pIconSchemaUsr = 0;

QPixmap * KPGTreeItem::m_pIconTable = 0;
QPixmap * KPGTreeItem::m_pIconView = 0;
QPixmap * KPGTreeItem::m_pIconSequence = 0;
QPixmap * KPGTreeItem::m_pIconOperator = 0;
QPixmap * KPGTreeItem::m_pIconOperatorClass = 0;
QPixmap * KPGTreeItem::m_pIconAggregate = 0;
QPixmap * KPGTreeItem::m_pIconFunction = 0;
QPixmap * KPGTreeItem::m_pIconFunctionC = 0;
QPixmap * KPGTreeItem::m_pIconFunctionSql = 0;
QPixmap * KPGTreeItem::m_pIconFunctionPl = 0;
QPixmap * KPGTreeItem::m_pIconUser = 0;
QPixmap * KPGTreeItem::m_pIconUserSuper = 0;
QPixmap * KPGTreeItem::m_pIconUserCanCreateDb = 0;
QPixmap * KPGTreeItem::m_pIconGroup = 0;
QPixmap * KPGTreeItem::m_pIconType = 0;
QPixmap * KPGTreeItem::m_pIconTypeBase = 0;
QPixmap * KPGTreeItem::m_pIconTypeComposite = 0;
QPixmap * KPGTreeItem::m_pIconTypeStandalone = 0;
QPixmap * KPGTreeItem::m_pIconLanguage = 0;
QPixmap * KPGTreeItem::m_pIconDomain = 0;
QPixmap * KPGTreeItem::m_pIconConversion = 0;
QPixmap * KPGTreeItem::m_pIconCast = 0;

QPixmap * KPGTreeItem::m_pIconPrimaryKeyConstr = 0;
QPixmap * KPGTreeItem::m_pIconForeignKeyConstr = 0;
QPixmap * KPGTreeItem::m_pIconUniqueConstr = 0;
QPixmap * KPGTreeItem::m_pIconCheckConstr = 0;
QPixmap * KPGTreeItem::m_pIconPrimaryKey = 0;
QPixmap * KPGTreeItem::m_pIconIndex = 0;
QPixmap * KPGTreeItem::m_pIconRule = 0;
QPixmap * KPGTreeItem::m_pIconTrigger = 0;

QPixmap * KPGTreeItem::m_pIconColumnOther = 0; 
QPixmap * KPGTreeItem::m_pIconColumnBool = 0; 
QPixmap * KPGTreeItem::m_pIconColumnGeo = 0;   
QPixmap * KPGTreeItem::m_pIconColumnNet = 0; 
QPixmap * KPGTreeItem::m_pIconColumnOid = 0;  
QPixmap * KPGTreeItem::m_pIconColumnXid = 0;
QPixmap * KPGTreeItem::m_pIconColumnTid = 0;
QPixmap * KPGTreeItem::m_pIconColumnCid = 0;
QPixmap * KPGTreeItem::m_pIconColumnDate = 0; 
QPixmap * KPGTreeItem::m_pIconColumnChar = 0;  
QPixmap * KPGTreeItem::m_pIconColumnNum = 0;
QPixmap * KPGTreeItem::m_pIconColumnTime = 0;
QPixmap * KPGTreeItem::m_pIconColumnBit = 0;
QPixmap * KPGTreeItem::m_pIconColumnBin = 0;

QPixmap * KPGTreeItem::m_pIconFolderBlue = 0;
QPixmap * KPGTreeItem::m_pIconFolderRed = 0;
QPixmap * KPGTreeItem::m_pIconFolderCyan = 0;
QPixmap * KPGTreeItem::m_pIconFolderGreen = 0;
QPixmap * KPGTreeItem::m_pIconFolderYellow = 0;
QPixmap * KPGTreeItem::m_pIconFolderViolet = 0;
QPixmap * KPGTreeItem::m_pIconFolderGrey = 0;


KPGTreeItem::KPGTreeItem(QListView *parent)
  : QListViewItem(parent)
{
}

KPGTreeItem::KPGTreeItem(KPGTreeItem *parent, const QString name)
  : QListViewItem(parent, name)
{
}

KPGTreeItem::KPGTreeItem(QListView *parent, KPGTreeItem *after)
  : QListViewItem(parent, after)
{
}

KPGTreeItem::KPGTreeItem(KPGTreeItem *parent, KPGTreeItem *after, const QString name)
  : QListViewItem(parent, after, name)
{
}

KPGTreeItem::~KPGTreeItem()
{
}

// Search object with given path and OID
KPGTreeItem * KPGTreeItem::searchObject(const QString &strPath, pqxx::oid _oid)
{
	int iPos = strPath.find('/');
	if(iPos <= 0) 
		{
			// path contain only one item name, it must match
		  if(strPath == text(0))
				return this;
			else
				return 0;
	  }
		
	// strip name and rest of path 
	QString strName = strPath.mid(0, iPos);
	if(strName != text(0))
		return 0; // This item is not on given path
	
	// This item is on given path, search in childs	
	// remove this name from path
	QString strPath2 = strPath.mid(iPos + 1, strPath.length() - iPos - 1);
									
	//--- traverse list of childs and search item here
	KPGTreeItem *pChildItem = static_cast <KPGTreeItem *> (firstChild());
				
	while(pChildItem)
	{
			KPGTreeItem * pFoundItem = 0;
				
			if((pFoundItem = pChildItem->searchObject(strPath2, _oid)) != 0)
					return pFoundItem;
	
			pChildItem = static_cast <KPGTreeItem *> (pChildItem->nextSibling());
	}			
	
	
	return 0;	
}

const QString KPGTreeItem::path()
{
	QString strPath(text(0));
	
	KPGTreeItem *pParent = this;
	while((pParent = static_cast <KPGTreeItem *> (pParent->parent())) != 0)
	{
		strPath.prepend(pParent->text(0) + "/");
	}
	
	return strPath;
}

QString KPGTreeItem::makeConnectionString(const QString _host, const QString _port, const QString _dbName, const QString _user, const QString _password)
{
			return QString("host=" + _host +
		               " port=" + _port +
									 " dbname=" + _dbName +
							     " user=" + _user +
									 " password=" + _password);
}

QString KPGTreeItem::makeConnectionString(const QString _host, const QString _port, const QString _user, const QString _password)
{
			return QString("host=" + _host +
		               " port=" + _port +
									 " user=" + _user +
									 " password=" + _password);
}

// Return connection to database. All items childs to KPGDatabase tells your parents for connection
KPGConnection * KPGTreeItem::connection()
{
	KPGTreeItem * pItemParent = static_cast <KPGTreeItem *> (parent());
	if(!pItemParent)
		{ 
		  kdError() << "Failed to find DB connection " << endl;
			return 0;
		}
						 
	// KPGTreeItem::connection() is redefined in KPGDatabase and return m_pDbConnection here
	return pItemParent->connection();
}

int KPGTreeItem::fillPropertiesTable(pqxx::result pqxxResult, QTable *pTable)
{
			
	// Fill output area with query result
  int nRows = pqxxResult.size();
  int nCols = pqxxResult.columns();
	
	// Compute table columns. OIDs, that aren't 1.st in tuple are NOT displaied
	int nTableCols = 0;
	int nCol;
	for(nCol = 0; nCol < nCols; nCol++)
    {
      if(/*(nCol == 0) && */ (pqxxResult.column_type(nCol) == 26)) // OID ?
			{
				; // do not count OID, if it is 1.st in result
			}
			else
			{
				nTableCols++;
			}
    }
	
	// Set table size
	pTable->setNumRows(nRows);
  pTable->setNumCols(nTableCols);

	// fill table with result data
	for(int nRow = 0; nRow < nRows; nRow++)
  {
    int nTableCol = 0;
		for(nCol = 0; nCol < nCols; nCol++)
    {
      if((nCol == 0) && (pqxxResult.column_type(nCol) == 26)) // OID ?
			{
				pqxx::oid oid = 0;
				pqxxResult[nRow][nCol].to(oid);
				nCol++;
				
				KPGLinkLabel *pLabel = new KPGLinkLabel( pTable, QString("%1").arg(oid) );
				pLabel->setValues(oid, pqxxResult[nRow][nCol].c_str());
				pTable->setCellWidget(nRow, nTableCol++, pLabel);
			}
			else
			{
				if(pqxxResult.column_type(nCol) != 26)
				{
					QString strValue(pqxxResult[nRow][nCol].c_str());
      		pTable->setText(nRow, nTableCol++, strValue);
				}
			}
    }
  }

	return nTableCols;
}

// Return child item with given name
KPGTreeItem * KPGTreeItem::getChildByName(const QString & strName)
{
	QListViewItem * pChildItem = firstChild();
	while(pChildItem)
	{			
		if(pChildItem->text(0) == strName)
			return static_cast <KPGTreeItem *> (pChildItem);
		
		pChildItem = pChildItem->nextSibling();
	}
	return 0;
}

// Return nearest parent item with given type
KPGTreeItem * KPGTreeItem::getParentByType(const eNodeType type)
{
  QListViewItem * pParentItem = parent();
  if(pParentItem)
  {
    if(static_cast <KPGTreeItem *> (pParentItem)->type() == type)
      return static_cast <KPGTreeItem *> (pParentItem);
    else
      return (static_cast <KPGTreeItem *> (pParentItem))->getParentByType(type);
  }
   
  return 0;
}

==> kpogre-1.5.2.tar.gz: kpogre-1.5.2/kpogre/DbObjects/kpgtreeitem.cpp <==
/***************************************************************************
                          kpgtreeitem.cpp  -  description
                             -------------------
    begin                : Ne led 11 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgtreeitem.h"

#include <qtable.h>

#include <kiconloader.h>
#include <kdebug.h>

#include "../kpglinklabel.h"

QPixmap * KPGTreeItem::m_pIconServer = 0;
QPixmap * KPGTreeItem::m_pIconTablespaceSys = 0;
QPixmap * KPGTreeItem::m_pIconTablespaceUsr = 0;
QPixmap * KPGTreeItem::m_pIconDatabase = 0;
QPixmap * KPGTreeItem::m_pIconDatabaseSys = 0;
QPixmap * KPGTreeItem::m_pIconDatabaseUsr = 0;
QPixmap * KPGTreeItem::m_pIconSchema = 0;
QPixmap * KPGTreeItem::m_pIconSchemaUnl = 0;
QPixmap * KPGTreeItem::m_pIconSchemaSys = 0;
QPixmap * KPGTreeItem::m_pIconSchemaUsr = 0;

QPixmap * KPGTreeItem::m_pIconTable = 0;
QPixmap * KPGTreeItem::m_pIconView = 0;
QPixmap * KPGTreeItem::m_pIconSequence = 0;
QPixmap * KPGTreeItem::m_pIconOperator = 0;
QPixmap * KPGTreeItem::m_pIconOperatorClass = 0;
QPixmap * KPGTreeItem::m_pIconAggregate = 0;
QPixmap * KPGTreeItem::m_pIconFunction = 0;
QPixmap * KPGTreeItem::m_pIconFunctionC = 0;
QPixmap * KPGTreeItem::m_pIconFunctionSql = 0;
QPixmap * KPGTreeItem::m_pIconFunctionPl = 0;
QPixmap * KPGTreeItem::m_pIconUser = 0;
QPixmap * KPGTreeItem::m_pIconUserSuper = 0;
QPixmap * KPGTreeItem::m_pIconUserCanCreateDb = 0;
QPixmap * KPGTreeItem::m_pIconGroup = 0;
QPixmap * KPGTreeItem::m_pIconType = 0;
QPixmap * KPGTreeItem::m_pIconTypeBase = 0;
QPixmap * KPGTreeItem::m_pIconTypeComposite = 0;
QPixmap * KPGTreeItem::m_pIconTypeStandalone = 0;
QPixmap * KPGTreeItem::m_pIconLanguage = 0;
QPixmap * KPGTreeItem::m_pIconDomain = 0;
QPixmap * KPGTreeItem::m_pIconConversion = 0;
QPixmap * KPGTreeItem::m_pIconCast = 0;

QPixmap * KPGTreeItem::m_pIconPrimaryKeyConstr = 0;
QPixmap * KPGTreeItem::m_pIconForeignKeyConstr = 0;
QPixmap * KPGTreeItem::m_pIconUniqueConstr = 0;
QPixmap * KPGTreeItem::m_pIconCheckConstr = 0;
QPixmap * KPGTreeItem::m_pIconPrimaryKey = 0;
QPixmap * KPGTreeItem::m_pIconIndex = 0;
QPixmap * KPGTreeItem::m_pIconRule = 0;
QPixmap * KPGTreeItem::m_pIconTrigger = 0;

QPixmap * KPGTreeItem::m_pIconColumnOther = 0; 
QPixmap * KPGTreeItem::m_pIconColumnBool = 0; 
QPixmap * KPGTreeItem::m_pIconColumnGeo = 0;   
QPixmap * KPGTreeItem::m_pIconColumnNet = 0; 
QPixmap * KPGTreeItem::m_pIconColumnOid = 0;  
QPixmap * KPGTreeItem::m_pIconColumnXid = 0;
QPixmap * KPGTreeItem::m_pIconColumnTid = 0;
QPixmap * KPGTreeItem::m_pIconColumnCid = 0;
QPixmap * KPGTreeItem::m_pIconColumnDate = 0; 
QPixmap * KPGTreeItem::m_pIconColumnChar = 0;  
QPixmap * KPGTreeItem::m_pIconColumnNum = 0;
QPixmap * KPGTreeItem::m_pIconColumnTime = 0;
QPixmap * KPGTreeItem::m_pIconColumnBit = 0;
QPixmap * KPGTreeItem::m_pIconColumnBin = 0;

QPixmap * KPGTreeItem::m_pIconFolderBlue = 0;
QPixmap * KPGTreeItem::m_pIconFolderRed = 0;
QPixmap * KPGTreeItem::m_pIconFolderCyan = 0;
QPixmap * KPGTreeItem::m_pIconFolderGreen = 0;
QPixmap * KPGTreeItem::m_pIconFolderYellow = 0;
QPixmap * KPGTreeItem::m_pIconFolderViolet = 0;
QPixmap * KPGTreeItem::m_pIconFolderGrey = 0;

QPixmap * KPGTreeItem::m_pIconNull = 0;

KPGTreeItem::KPGTreeItem(QListView *parent)
  : QListViewItem(parent)
{
}

KPGTreeItem::KPGTreeItem(KPGTreeItem *parent, const QString name)
  : QListViewItem(parent, name)
{
}

KPGTreeItem::KPGTreeItem(QListView *parent, KPGTreeItem *after)
  : QListViewItem(parent, after)
{
}

KPGTreeItem::KPGTreeItem(KPGTreeItem *parent, KPGTreeItem *after, const QString name)
  : QListViewItem(parent, after, name)
{
}

KPGTreeItem::~KPGTreeItem()
{
}

// Search object with given path and OID
KPGTreeItem * KPGTreeItem::searchObject(const QString &strPath, pqxx::oid _oid)
{
	int iPos = strPath.find('/');
	if(iPos <= 0) 
		{
			// path contain only one item name, it must match
		  if(strPath == text(0))
				return this;
			else
				return 0;
	  }
		
	// strip name and rest of path 
	QString strName = strPath.mid(0, iPos);
	if(strName != text(0))
		return 0; // This item is not on given path
	
	// This item is on given path, search in childs	
	// remove this name from path
	QString strPath2 = strPath.mid(iPos + 1, strPath.length() - iPos - 1);
									
	//--- traverse list of childs and search item here
	KPGTreeItem *pChildItem = static_cast <KPGTreeItem *> (firstChild());
				
	while(pChildItem)
	{
			KPGTreeItem * pFoundItem = 0;
				
			if((pFoundItem = pChildItem->searchObject(strPath2, _oid)) != 0)
					return pFoundItem;
	
			pChildItem = static_cast <KPGTreeItem *> (pChildItem->nextSibling());
	}			
	
	
	return 0;	
}

const QString KPGTreeItem::path()
{
	QString strPath(text(0));
	
	KPGTreeItem *pParent = this;
	while((pParent = static_cast <KPGTreeItem *> (pParent->parent())) != 0)
	{
		strPath.prepend(pParent->text(0) + "/");
	}
	
	return strPath;
}

const QString KPGTreeItem::makeConnectionString(const QString &strHost, 
	const QString &strPort, 
	const QString &strDbName, 
	const QString &strUser, 
	const QString &strPassword)
{
	return QString("host=" + strHost +
	               " port=" + strPort +
					 " dbname=" + strDbName +
				     " user=" + strUser +
					 " password=" + strPassword);
}

const QString KPGTreeItem::makeConnectionString(const QString &strHost, 
	const QString &strPort, 
	const QString &strUser, 
	const QString &strPassword)
{
	return QString("host=" + strHost +
		            " port=" + strPort +
					" user=" + strUser +
					" password=" + strPassword);
}

// Return connection to database. All items childs to KPGDatabase tells your parents for connection
KPGConnection * KPGTreeItem::connection()
{
	KPGTreeItem * pItemParent = static_cast <KPGTreeItem *> (parent());
	if(!pItemParent)
	{ 
		kdError() << k_funcinfo << "Failed to find DB connection " << endl;
		return 0;
	}
						 
	// KPGTreeItem::connection() is redefined in KPGDatabase and return m_pDbConnection here
	return pItemParent->connection();
}

int KPGTreeItem::fillPropertiesTable(pqxx::result pqxxResult, QTable *pTable, bool bAllowOidOnlyIn1stCol, int nMaxColumn)
{
	// Fill output area with query result
	int nRows = pqxxResult.size();
	int nCols = pqxxResult.columns();
	
	// Compute table columns. OIDs, that aren't 1.st in tuple are NOT displaied
	int nTableCols = 0;
	int nCol;
  	for(nCol = 0; nCol < nCols; nCol++)
  	{
  		if(pqxxResult.column_type(nCol) == 26) // OID ?
	  	{ ; /* do not count OIDs */ }
		else
		{
			nTableCols++;
		}
		
		if((nMaxColumn > 0) && (nTableCols >= nMaxColumn))
			break;
  	}
	
	// Set table size
	pTable->setNumRows(nRows);
  	pTable->setNumCols(nTableCols);

	// fill table with result data
	for(int nRow = 0; nRow < nRows; nRow++)
  	{
    	int nCol = 0;
		for(int nTableCol = 0; nTableCol < nTableCols; nTableCol++)
		{
			if((!bAllowOidOnlyIn1stCol || (nCol == 0)) && (pqxxResult.column_type(nCol) == 26)) // OID ?
			{
				pqxx::oid oid = 0;
				pqxxResult[nRow][nCol].to(oid);
				nCol++;
					
				KPGLinkLabel *pLabel = new KPGLinkLabel( pTable/*, QString("%1").arg(oid) */);
				pLabel->setValues(oid, pqxxResult[nRow][nCol].c_str());
				pTable->setCellWidget(nRow, nTableCol, pLabel);
			}
			else
			{
				if(pqxxResult.column_type(nCol) != 26)
				{
					QString strValue(pqxxResult[nRow][nCol].c_str());
					pTable->setText(nRow, nTableCol, strValue);
				}
				else
				{
				    nCol++; // skip OID
				    
				    QString strValue(pqxxResult[nRow][nCol].c_str());
					pTable->setText(nRow, nTableCol, strValue);
				}
			}
			nCol++;
		}
  	}

	return nTableCols;
}

// Return child item with given name
KPGTreeItem * KPGTreeItem::getChildByName(const QString & strName)
{
	//kdDebug() << "Finding: " << strName << endl;
	
	QListViewItem * pChildItem = firstChild();
	while(pChildItem)
	{	
		//kdDebug() << "Comparing with: " << pChildItem->text(0) << endl;
				
		if(pChildItem->text(0) == strName)
			return static_cast <KPGTreeItem *> (pChildItem);
		
		pChildItem = pChildItem->nextSibling();
	}
	return 0;
}

// Return nearest parent item with given type
KPGTreeItem * KPGTreeItem::getParentByType(const eNodeType type)
{
	QListViewItem * pParentItem = parent();
	if(pParentItem)
	{
		if(static_cast <KPGTreeItem *> (pParentItem)->type() == type)
			return static_cast <KPGTreeItem *> (pParentItem);
		else
			return (static_cast <KPGTreeItem *> (pParentItem))->getParentByType(type);
	}
	
	return 0;
}  

// Sort the KPGOidNameList
void KPGOidNameList::sort()
{
    qHeapSort( *this );
}

const KPGOidName * KPGOidNameList::getItemByName(const QString &strName) const
{
    for(KPGOidNameList::const_iterator cit = begin(); cit != end(); ++cit)
	{
        if((*cit).name() == strName)
        {
            const KPGOidName &oidName = (*cit);
            return &oidName;
        }
    }
    
    return 0;
}
